use anyhow::Context;
use clap::arg;
use prymn_agent::{self_update, server};

fn main() -> anyhow::Result<()> {
    tracing_subscriber::fmt().init();

    let command = clap::Command::new(env!("CARGO_BIN_NAME"))
        .version(env!("CARGO_PKG_VERSION"))
        .arg(arg!(-d --daemon "Run agent as daemon").exclusive(true))
        .arg(arg!(--install <TOKEN> "Install this agent binary to the system").exclusive(true))
        .arg_required_else_help(true)
        .try_get_matches()
        .unwrap_or_else(|e| e.exit());

    if command.get_flag("daemon") {
        tracing::info!("starting agent");
        server::main()
    } else if let Some(token) = command.get_one::<String>("install") {
        tracing::info!("starting installation...");
        self_update::install(token).context("failed to install the agent to the system")
    } else {
        unreachable!()
    }
}
