defmodule PrymnWeb.ServerLive.Index do
  require Logger
  alias Prymn.{Servers, Agents}

  use PrymnWeb, :live_view

  @impl true
  def mount(_params, _session, socket) do
    servers = Servers.list_servers()
    # pid = self()

    for %Servers.Server{status: :registered, public_ip: ip} <- servers do
      :ok = Phoenix.PubSub.subscribe(Prymn.PubSub, "agent:#{ip}")

      Task.start_link(fn ->
        case Agents.ensure_connection(ip) do
          :ok -> IO.puts("Ok")
          _ -> IO.puts("not ok")
        end
      end)
    end

    {:ok, assign(socket, :servers, servers)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    {:noreply, apply_action(socket, socket.assigns.live_action, params)}
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Server")
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Servers")
  end

  @impl true
  def handle_info({:connect, %Servers.Server{} = server}, socket) do
    {:noreply,
     socket
     |> update(:servers, fn servers -> [server | servers] end)}
  end

  @impl true
  def handle_info({:healthcheck, ip, _message}, socket) do
    servers =
      update_in(
        socket.assigns.servers,
        [Access.filter(&match?(%{public_ip: ^ip}, &1))],
        &Map.merge(&1, %{connection_status: "Connected"})
      )

    {:noreply, assign(socket, :servers, servers)}
  end

  @impl true
  def handle_info(msg, state) do
    Logger.debug("received unexpected message #{inspect(msg)}")
    {:noreply, state}
  end

  # @impl true
  # def handle_event("delete", %{"id" => id}, socket) do
  #   server = Servers.get_server!(id)
  #   {:ok, _} = Servers.delete_server(server)

  #   {:noreply, stream_delete(socket, :servers, server)}
  # end
end
